namespace :search do
  desc 'search learning objects'
  task :learning_objects, [:filename] => :environment do |_t, args|
    queries = %w(dna molecula floresta matematica)
    objects = queries.map { |q| mount_search(q, 'LearningObject') }
    File.open("#{Rails.root}/db/backups/#{args.filename}_searches.json", 'w+') do |f|
      f.write(JSON.pretty_generate(objects))
    end
  end

  private

  def mount_search(query, search_class)
    search = SearchService.instance(Search.new(query: query, search_class: search_class), nil).search
    {
      query: query,
      results: {
        elastic: mount_result(LearningObject.search(query)),
        reputation: mount_result(LearningObject.search(query, order: { score: { order: :desc, unmapped_type: :integer } })),
        merged: mount_result(search)
      }
    }
  end

  def mount_result(search, count = 50)
    scores = Score.where('active = true AND score_type @> \'{LearningObject}\'')
    objects = search.results.first(count).each_with_index.map do |object, index|
      # score dont exists in ordered searches, so we use sort field
      score = search.response['hits']['hits'][index]['sort'] || search.response['hits']['hits'][index]['_score']
      o = { object: object.inspect, score: score, reputation_scores: [] }
      scores.each do |s|
        o[:reputation_scores] << { name: s.name, weight: s.weight, value: object.try(s.code.to_sym) }
      end
      o
    end

    {
      hits: search.response['hits']['total'],
      objects: objects
    }
  end
end
